﻿using System;
using System.Security.Cryptography;
using System.Text;

namespace KellermanSoftware.NetFtpLibraryCoreDemo
{
    internal sealed class Encryptor
    {
        public void Encrypt(Configuration configuration)
        {
            configuration.Server = Encrypt(configuration.Server);
            configuration.Username = Encrypt(configuration.Username);
            configuration.Password = Encrypt(configuration.Password);
            configuration.LicenseUsername = Encrypt(configuration.LicenseUsername);
            configuration.LicenseKey = Encrypt(configuration.LicenseKey);
        }

        public void Decrypt(Configuration configuration)
        {
            configuration.Server = Decrypt(configuration.Server);
            configuration.Username = Decrypt(configuration.Username);
            configuration.Password = Decrypt(configuration.Password);
            configuration.LicenseUsername = Decrypt(configuration.LicenseUsername);
            configuration.LicenseKey = Decrypt(configuration.LicenseKey);
        }

        private static string Encrypt(string text)
        {
            using (var algorithm = CreateAlgorithm())
            using (var encryptor = algorithm.CreateEncryptor())
            {
                var inputBytes = Encoding.Unicode.GetBytes(text);
                var outputBytes = encryptor.TransformFinalBlock(inputBytes, 0, inputBytes.Length);
                return Convert.ToBase64String(outputBytes);
            }
        }

        private static string Decrypt(string text)
        {
            using (var algorithm = CreateAlgorithm())
            using (var decryptor = algorithm.CreateDecryptor())
            {
                var inputBytes = Convert.FromBase64String(text);
                var outputBytes = decryptor.TransformFinalBlock(inputBytes, 0, inputBytes.Length);
                return Encoding.Unicode.GetString(outputBytes);
            }
        }

        private static Aes CreateAlgorithm()
        {
            var algorithm = Aes.Create();
            using (var generator = new Rfc2898DeriveBytes(Password, Salt, 1000))
            {
                algorithm.Key = generator.GetBytes(32);
                algorithm.IV = generator.GetBytes(16);
            }
            return algorithm;
        }

        private static string Password
        {
            get
            {
                return Environment.MachineName;
            }
        }

        private static byte[] Salt
        {
            get
            {
                using (var algorithm = SHA256.Create())
                {
                    return algorithm.ComputeHash(Encoding.Unicode.GetBytes(Password));
                }
            }
        }
    }
}
