﻿using KellermanSoftware.NetFtpLibrary;
using System;
using System.IO;

namespace KellermanSoftware.NetFtpLibraryCoreDemo
{
    internal sealed class FtpClient : IDisposable
    {
        private readonly FTP _ftp;

        public FtpClient(Configuration configuration)
        {
            _ftp = new FTP(configuration.LicenseUsername, configuration.LicenseKey)
            {
                HostAddress = configuration.Server,
                UserName = configuration.Username,
                Password = configuration.Password,
                EnableSSL = configuration.UseSsl
            };
        }

        public void Dispose()
        {
            if (_ftp != null)
            {
                _ftp.Dispose();
            }
        }

        public void Upload(string source, string destination)
        {
            if (File.Exists(source))
            {
                UploadFile(source, destination);
            }
            else if (Directory.Exists(source))
            {
                UploadDirectory(source, destination);
            }
            else
            {
                throw new FileNotFoundException($"Cannot find source '{source}'");
            }
        }

        public void Download(string source, string destination)
        {
            try
            {
                _ftp.Connect();
                if (_ftp.FileExists(source))
                {
                    _ftp.DownloadFile(destination, source);
                }
                else if (_ftp.DirectoryExists(source))
                {
                    _ftp.DownloadDirectory(destination, source);
                }
                else
                {
                    throw new FileNotFoundException($"Cannot find source '{source}'");
                }
            }
            finally
            {
                _ftp.Disconnect();
            }
        }

        private void UploadFile(string sourceFile, string destinationFile)
        {
            try
            {
                _ftp.Connect();
                _ftp.UploadFile(sourceFile, destinationFile);
            }
            finally
            {
                _ftp.Disconnect();
            }
        }

        private void UploadDirectory(string sourceDirectory, string destinationDirectory)
        {
            try
            {
                _ftp.Connect();
                _ftp.UploadDirectory(sourceDirectory, destinationDirectory);
            }
            finally
            {
                _ftp.Disconnect();
            }
        }
    }
}
