using System;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using KellermanSoftware.NetFtpLibrary;

namespace KellermanSoftware.NetFtpLibraryDemo
{

    delegate void ProgressEvent(object sender, TransferProgressChangedEventArgs e);
    delegate void CompleteEvent(object sender, TransferCompletedEventArgs e);

    public partial class ProgressForm : Form
    {
        public FTP _ftp;
        public bool isDownloading;
        private bool cancelled = true;
        private string curFile;
        private bool _closing = false;

        public ProgressForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Fired when the transfer completes, it closes the form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void ftp_TransferCompleteEvent(object sender, TransferCompletedEventArgs e)
        {
            try
            {
                if (this.InvokeRequired)
                {
                    CompleteEvent prog = new CompleteEvent(ftp_TransferCompleteEvent);
                    this.Invoke(prog, new object[] { sender, e });
                    return;
                }
                cancelled = false;
                this.Close();
            }
            catch
            {
            }
        }

        /// <summary>
        /// Updates the GUI during an async transfer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void ftp_ProgressEvent(object sender, TransferProgressChangedEventArgs e)
        {
            if (Disposing || _closing)
            {
                return;
            }

            if (this.InvokeRequired)
            {
                ProgressEvent prog = new ProgressEvent(ftp_ProgressEvent);
                this.Invoke(prog, new object[] { (FTP)sender, e });
                return;
            }

            curFile = _ftp.CurrentFileBeingTransferred;
            if (isDownloading)
                curFile = _ftp.CurrentFileBeingTransferredDestination + Path.DirectorySeparatorChar + curFile;

            groupBox1.Text = "Transfering: " + curFile;
            if (e.TotalBytesToTransfer > 0)
            {
                progressBar.Value = e.ProgressPercentage;
                lblTimeRemaining.Text = _ftp.EstimatedTimeRemaining.ToString();
                lblRemaining.Text = (e.TotalBytesToTransfer - e.BytesTransferred).ToString();
            }

            lblAvg.Text = _ftp.KiloByteAverage.ToString("n" + 2);
            lblCurrent.Text = _ftp.KiloBytePerSecond.ToString("n" + 2);
            lblElapsed.Text = _ftp.TimeElapsed.ToString();
            lblPeak.Text = _ftp.KiloBytePeak.ToString("n" + 2);
            lblTransfered.Text = e.BytesTransferred.ToString();
            lblIsBusy.Text = _ftp.IsBusy.ToString();
        }

        /// <summary>
        /// Fired when the form is closing, it cancels the async trasnfer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Form2_FormClosing(object sender, System.Windows.Forms.FormClosingEventArgs e)
        {
            _closing = true;
            if (cancelled)
            {
                _ftp.Cancel();
                try
                {
                    // delete partial file
                    if (isDownloading)
                    {
                        // local file to delete
                        Thread.Sleep(500);
                        //File.Delete(curFile);
                    }
                    else
                    {
                        // remote file to delete
                        //_ftp.DeleteFile(curFile);
                    }
                }
                catch (Exception)
                {
                }
            }
        }

        /// <summary>
        /// Closes the form and cancels the async transfer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}